@@ -99,26 +99,25 @@ def build(input_graph):
     g_phy = anm['phy']
 
     build_vrf(anm)
+    build_ip(anm) # ip infrastructure topology
+    autonetkit.update_http(anm)
 
-    address_family = g_in.data.address_family or "v4"
-    allocate_ipv4_infrastructure = False
+    address_family = g_in.data.address_family or "v4" # default is v4
+#TODO: can remove the infrastructure now create g_ip seperately
     if address_family in ("v4", "dual_stack"):
-        allocate_ipv4_infrastructure = True
-    build_ipv4(anm, infrastructure=allocate_ipv4_infrastructure)
+        build_ipv4(anm, infrastructure = True)
+        g_phy.update(g_phy, use_ipv4 = True)
+    else:
+        build_ipv4(anm, infrastructure = False)
 
     #TODO: Create a collision domain overlay for ip addressing - l2 overlay?
-    allocate_ipv6 = False
     if address_family in ("v6", "dual_stack"):
-        allocate_ipv6 = True
-        build_ipv4(anm, infrastructure=True)
-
-    build_ipv6(anm)
-    for node in g_phy:
-        node.use_ipv4 = allocate_ipv4_infrastructure
-        node.use_ipv6 = allocate_ipv6
+        build_ipv6(anm)
+        g_phy.update(g_phy, use_ipv6 = True)
 
     default_igp = g_in.data.igp or "ospf" 
-    non_igp_nodes = [n for n in g_in if not node.igp]
+    non_igp_nodes = [n for n in g_in if not n.igp]
+#TODO: should this be modifying g_in?
     g_in.update(non_igp_nodes, igp=default_igp) # store igp onto each node
 
     anm.add_overlay("ospf")
@@ -245,7 +244,7 @@ def three_tier_ibgp_corner_cases(rtrs):
             elif all(r.ibgp_level == 1 for r in l2d):
                 # No l2 or l3 routers -> full-mesh of l1 routers
                 over_links += [(s, t) for s in l2d for t in l2d if s != t]
-                log.info("Cluster (%s, %s) has no level 2 or 3 iBGP routers."
+                log.debug("Cluster (%s, %s) has no level 2 or 3 iBGP routers."
                         "Connecting l1 routers (%s) in full-mesh"
                         % (l3_cluster, l2_cluster, l2d))
             else:
@@ -253,7 +252,7 @@ def three_tier_ibgp_corner_cases(rtrs):
                 l3_rtrs = [r for r in l2d if r.ibgp_level == 3]
                 if not(len(l1_rtrs) and len(l3_rtrs)):
                     break  # no routers to connect
-                log.info("Cluster (%s, %s) has no level 2 iBGP routers."
+                log.debug("Cluster (%s, %s) has no level 2 iBGP routers."
                           "Connecting l1 routers (%s) to l3 routers (%s)"
                           % (l3_cluster, l2_cluster, l1_rtrs, l3_rtrs))
 
@@ -431,12 +430,13 @@ def build_ipv6(anm):
     import autonetkit.plugins.ipv6 as ipv6
     # uses the nodes and edges from ipv4
     g_ipv6 = anm.add_overlay("ipv6")
-    g_ipv4 = anm['ipv4']
+    g_ip = anm['ip']
     g_ipv6.add_nodes_from(
-        g_ipv4, retain="collision_domain")  # retain if collision domain or not
-    g_ipv6.add_edges_from(g_ipv4.edges())
-    ipv6.allocate_ips(g_ipv6)
+        g_ip, retain="collision_domain")  # retain if collision domain or not
+    g_ipv6.add_edges_from(g_ip.edges())
+    autonetkit.update_http(anm)
 
+    ipv6.allocate_ips(g_ipv6)
 
     #TODO: replace this with direct allocation to interfaces in ip alloc plugin
     for node in g_ipv6.nodes("is_l3device"):
@@ -448,15 +448,14 @@ def build_ipv6(anm):
                 interface.ip_address = edge.ip #TODO: make this consistent
                 interface.subnet = edge.dst.subnet # from collision domain
 
-def manual_ipv4_allocation(anm):
+def manual_ipv4_infrastructure_allocation(anm):
     """Applies manual IPv4 allocation"""
     import netaddr
     g_in_directed = anm['input_directed']
     g_ipv4 = anm['ipv4']
+    #TODO: tidy this up to work with interfaces directly
 
     for l3_device in g_ipv4.nodes("is_l3device"):
-        directed_node = g_in_directed.node(l3_device)
-        l3_device.loopback = directed_node.ipv4loopback
         for edge in l3_device.edges():
             # find edge in g_in_directed
             directed_edge = g_in_directed.edge(edge)
@@ -478,80 +477,113 @@ def manual_ipv4_allocation(anm):
 
     # also need to form aggregated IP blocks (used for e.g. routing prefix
     # advertisement)
-    loopback_blocks = {}
     infra_blocks = {}
+    for asn, devices in g_ipv4.groupby("asn").items():
+        collision_domains = [d for d in devices if d.collision_domain]
+        subnets = [cd.subnet for cd in collision_domains]
+        infra_blocks[asn] = netaddr.cidr_merge(subnets)
+
+    g_ipv4.data.infra_blocks = infra_blocks
+
+def manual_ipv4_loopback_allocation(anm):
+    """Applies manual IPv4 allocation"""
+    import netaddr
+    g_in_directed = anm['input_directed']
+    g_ipv4 = anm['ipv4']
+
+    for l3_device in g_ipv4.nodes("is_l3device"):
+        directed_node = g_in_directed.node(l3_device)
+        l3_device.loopback = directed_node.ipv4loopback
+
+    # also need to form aggregated IP blocks (used for e.g. routing prefix
+    # advertisement)
+    loopback_blocks = {}
     for asn, devices in g_ipv4.groupby("asn").items():
         routers = [d for d in devices if d.is_router]
         loopbacks = [r.loopback for r in routers]
         loopback_blocks[asn] = netaddr.cidr_merge(loopbacks)
 
-        collision_domains = [d for d in devices if d.collision_domain]
-        subnets = [cd.subnet for cd in collision_domains]
-        infra_blocks[asn] = netaddr.cidr_merge(subnets)
-
     g_ipv4.data.loopback_blocks = loopback_blocks
-    g_ipv4.data.infra_blocks = infra_blocks
 
-def build_ipv4(anm, infrastructure=True):
-    """Builds IPv4 graph"""
-    g_ipv4 = anm.add_overlay("ipv4")
+def build_ip(anm):
+    g_ip = anm.add_overlay("ip")
     g_in = anm['input']
     g_graphics = anm['graphics']
     g_phy = anm['phy']
 
-    g_ipv4.add_nodes_from(g_in)
-    g_ipv4.add_edges_from(g_in.edges(type="physical"))
+    g_ip.add_nodes_from(g_in)
+    g_ip.add_edges_from(g_in.edges(type="physical"))
 
-    ank_utils.aggregate_nodes(g_ipv4, g_ipv4.nodes("is_switch"),
+    ank_utils.aggregate_nodes(g_ip, g_ip.nodes("is_switch"),
                               retain="edge_id")
 
-    edges_to_split = [edge for edge in g_ipv4.edges() if edge.attr_both(
+    edges_to_split = [edge for edge in g_ip.edges() if edge.attr_both(
         "is_l3device")]
     for edge in edges_to_split:
         edge.split = True # mark as split for use in building nidb
     split_created_nodes = list(
-        ank_utils.split(g_ipv4, edges_to_split, retain=['edge_id', 'split']))
+        ank_utils.split(g_ip, edges_to_split, retain=['edge_id', 'split']))
     for node in split_created_nodes:
-        node['graphics'].x = ank_utils.neigh_average(g_ipv4, node, "x",
+        node['graphics'].x = ank_utils.neigh_average(g_ip, node, "x",
                                                      g_graphics) + 0.1 # temporary fix for gh-90
-        node['graphics'].y = ank_utils.neigh_average(g_ipv4, node, "y",
+        node['graphics'].y = ank_utils.neigh_average(g_ip, node, "y",
                                                      g_graphics) + 0.1 # temporary fix for gh-90
         asn = ank_utils.neigh_most_frequent(
-            g_ipv4, node, "asn", g_phy)  # arbitrary choice
+            g_ip, node, "asn", g_phy)  # arbitrary choice
         node['graphics'].asn = asn
         node.asn = asn # need to use asn in IP overlay for aggregating subnets
 
-    switch_nodes = g_ipv4.nodes("is_switch")  # regenerate due to aggregated
-    g_ipv4.update(switch_nodes, collision_domain=True)
+    switch_nodes = g_ip.nodes("is_switch")  # regenerate due to aggregated
+    g_ip.update(switch_nodes, collision_domain=True)
                  # switches are part of collision domain
-    g_ipv4.update(split_created_nodes, collision_domain=True)
+    g_ip.update(split_created_nodes, collision_domain=True)
 # Assign collision domain to a host if all neighbours from same host
     for node in split_created_nodes:
-        if ank_utils.neigh_equal(g_ipv4, node, "host", g_phy):
+        if ank_utils.neigh_equal(g_ip, node, "host", g_phy):
             node.host = ank_utils.neigh_attr(
-                g_ipv4, node, "host", g_phy).next()  # first attribute
+                g_ip, node, "host", g_phy).next()  # first attribute
 
 # set collision domain IPs
-    for node in g_ipv4.nodes("collision_domain"):
+    for node in g_ip.nodes("collision_domain"):
         graphics_node = g_graphics.node(node)
         graphics_node.device_type = "collision_domain"
         if not node.is_switch:
             label = "_".join(
-                sorted(ank_utils.neigh_attr(g_ipv4, node, "label", g_phy)))
+                sorted(ank_utils.neigh_attr(g_ip, node, "label", g_phy)))
             cd_label = "cd_%s" % label  # switches keep their names
             node.label = cd_label
             node.cd_id = cd_label
             graphics_node.label = cd_label
 
+def build_ipv4(anm, infrastructure=True):
+    """Builds IPv4 graph"""
+    g_ipv4 = anm.add_overlay("ipv4")
+    g_ip = anm['ip']
+    g_in = anm['input']
+    g_ipv4.add_nodes_from(
+        g_ip, retain="collision_domain")  # retain if collision domain or not
+    # Copy ASN attribute chosen for collision domains (used in alloc algorithm)
+    ank_utils.copy_attr_from(g_ip, g_ipv4, "asn", nbunch = g_ipv4.nodes("collision_domain"))
+    g_ipv4.add_edges_from(g_ip.edges())
+    autonetkit.update_http(anm)
+
     #TODO: need to set allocate_ipv4 by default in the readers
-    if g_in.data.allocate_ipv4 is False:
-        manual_ipv4_allocation(anm)
+    if g_in.data.alloc_ipv4_infrastructure is False:
+        manual_ipv4_infrastructure_allocation(anm)
     else:
         import autonetkit.plugins.ipv4 as ipv4
-        ipv4.allocate_ips(g_ipv4, infrastructure)
-        ank_utils.save(g_ipv4)
+        ipv4.allocate_ips(g_ipv4, infrastructure = True, loopbacks = False)
+        #ank_utils.save(g_ipv4)
+
+    if g_in.data.alloc_ipv4_loopbacks is False:
+        manual_ipv4_loopback_allocation(anm)
+    else:
+        import autonetkit.plugins.ipv4 as ipv4
+        ipv4.allocate_ips(g_ipv4, infrastructure = False, loopbacks = True)
+        #ank_utils.save(g_ipv4)
 
     autonetkit.update_http(anm)
+
     #TODO: replace this with direct allocation to interfaces in ip alloc plugin
     for node in g_ipv4.nodes("is_l3device"):
         node.loopback_zero.ip_address = node.loopback
@@ -741,6 +773,9 @@ def ip_to_net_ent_title_ios(ip_addr):
 def build_isis(anm):
     """Build isis overlay"""
     g_in = anm['input']
+    if not any(n.igp == "isis" for n in g_in):
+        log.debug("No ISIS nodes")
+        return
     g_ipv4 = anm['ipv4']
     g_isis = anm.add_overlay("isis")
     g_isis.add_nodes_from(g_in.nodes("is_router", igp = "isis"), retain=['asn'])
