@@ -109,7 +109,7 @@ var apply_highlight = function(data){
         return {'source': src_index, 'target': target_index};
     }
     )
-    redraw();
+        redraw();
 }
 
 var load_ip_allocations = function(d) {
@@ -253,7 +253,7 @@ var propagate_revision_dropdown = function(d) {
     } else {
         $('#revision_select').hide();
     }        
-        
+
     revision_dropdown
         .selectAll("option")
         .data(revisions)
@@ -461,32 +461,33 @@ var groupPath = function(d) {
             retval += "Z";
             return retval
         } else if (d.values.length  == 2) {
-                interface1 = d.values[1];
-                interface2 = d.values[0];
-                //Note: upper_x goes with lower_y due to y ascending down, x ascending right
-                int1_x = interface_x(interface1);
-                int1_y = interface_y(interface1);
-                int2_x = interface_x(interface2);
-                int2_y = interface_y(interface2);
-                upper_x = Math.max(int1_x, int2_x);
-                upper_y = Math.max(int1_y, int2_y);
-                lower_x = Math.min(int1_x, int2_x);
-                lower_y = Math.min(int1_y, int2_y);
+            interface1 = d.values[1];
+            interface2 = d.values[0];
+            //Note: upper_x goes with lower_y due to y ascending down, x ascending right
+            int1_x = interface_x(interface1);
+            int1_y = interface_y(interface1);
+            int2_x = interface_x(interface2);
+            int2_y = interface_y(interface2);
+            upper_x = Math.max(int1_x, int2_x);
+            upper_y = Math.max(int1_y, int2_y);
+            lower_x = Math.min(int1_x, int2_x);
+            lower_y = Math.min(int1_y, int2_y);
 
-                offset = 0.1;
-                retval =  "M" ;
-                retval += (upper_x - offset ) + "," + (upper_y - offset ) + "L";
-                retval += (upper_x + offset ) + "," + (upper_y - offset ) + "L";
-                retval += (lower_x + offset ) + "," + (lower_y + offset ) + "L";
-                retval += (lower_x - offset ) + "," + (lower_y + offset ) ;
-                retval += "Z";
-                return retval;
+            offset = 0.1;
+            retval =  "M" ;
+            retval += (upper_x - offset ) + "," + (upper_y - offset ) + "L";
+            retval += (upper_x + offset ) + "," + (upper_y - offset ) + "L";
+            retval += (lower_x + offset ) + "," + (lower_y + offset ) + "L";
+            retval += (lower_x - offset ) + "," + (lower_y + offset ) ;
+            retval += "Z";
+            return retval;
         } else {
+            interface_offset = 5;
             retval = "M" + 
                 d3.geom.hull(d.values.map(function(i) { 
-                    return [interface_x(i), interface_y(i)]; 
+                    return [interface_x(i) + interface_offset, interface_y(i) + interface_offset]; 
                 }))
-                .join("L")
+            .join("L")
                 + "Z";
             return retval;
         }
@@ -610,7 +611,8 @@ var interface_info = function(d) {
     int_data = d.node._interfaces[d.interface];
 
     text = "<ul>"; //begin the unordered list
-    text += "<li><b>id:</b> " + d.interface + "</li>"; //add the key/val
+    text += "<li><b>node:</b> " + d.node.id + "</li>"; //add the key/val
+    text += "<li><b>interface:</b> " + d.interface + "</li>"; //add the key/val
     for (attr in int_data) {
         text += "<li><b>" + attr + ":</b> " + int_data[attr] + "</li>"; //add the key/val
     }
@@ -695,7 +697,7 @@ var graph_edge = function(d) {
     if (jsondata.directed) {
         var dx = target_x - source_x,
             dy = target_y - source_y,
-            dr = Math.sqrt(dx * dx + dy * dy);
+               dr = Math.sqrt(dx * dx + dy * dy);
         //dr = 1.2 * dr;
         //return "M" + source_x + "," + source_y + "A" + dr + "," + dr + " 0 0,1 " + target_x + "," + target_y;
         var points = [];
@@ -737,7 +739,7 @@ var directed_edge_offset_x = function(source, target, hypotenuse) {
     dr = Math.sqrt(dx * dx + dy * dy);
 
     hypotenuse = typeof hypotenuse !== 'undefined' ? hypotenuse : dr/4; //defaults to dr/4
-    
+
     angle = Math.atan2( (t_x - s_x), (t_y - s_y));
     angle = angle + alpha;
     offset_x = hypotenuse * Math.sin(angle);
@@ -912,8 +914,8 @@ var device_label = function(d) {
 
 var interface_label = function(d) {
     try {
-    int_data = d.node._interfaces[d.interface];
-    return int_data[interface_label_id];
+        int_data = d.node._interfaces[d.interface];
+        return int_data[interface_label_id];
     }
     catch (err) {
         //console.log(err);
@@ -933,9 +935,9 @@ var zoom_fit = function() {
         var zoom_box = d3.select(".zoom_box")
 
             zoom_box.transition()
-                    .attr("transform", "scale(" + p + ")")
-                   .duration(500)
-        //redraw();
+            .attr("transform", "scale(" + p + ")")
+            .duration(500)
+            //redraw();
     }
 }
 
@@ -984,6 +986,17 @@ var icon_opacity = function(x) {
     return 0.2; //drop opacity for non filtered
 };
 
+var interface_opacity = function(x) {
+    if (filtered_nodes.length == 0) return 1;
+    //no filtered, so display all at full opacity
+
+    if (_.contains(filtered_nodes, x.node)) 
+    {
+        return 1; 
+    }//some are filtered, full opacity for these
+    return 0.2; //drop opacity for non filtered
+};
+
 var line_opacity = function(x) {
     if (filtered_nodes.length == 0) return 1; //no filtered, so display all at full opacity
 
@@ -1001,9 +1014,9 @@ var edge_attributes = [];
 
 function redraw() {
     //TODO: tidy this up, not all functions need to be in here, move out those that do, and only pass required params. also avoid repeated calculations.
-    
+
     nodes = jsondata.nodes;
-        
+
     node_attributes = []; //reset
     nodes.forEach(function(node) {
         nodes_by_id[node.id] = node;
@@ -1089,7 +1102,7 @@ function redraw() {
     node_attr_groups = d3.nest().key( node_group_id ).entries(nodes);
     edge_attr_groups = d3.nest().key(function(d) { return d[edge_group_id]; }).entries(jsondata.links);
     //TODO: use edge attr groups for edge colours
-    
+
     //If undirected graph, then need two interfaces per edge: one at each end
     if (display_interfaces) {
         //Undirected, need to handle for both src and dst
@@ -1112,7 +1125,7 @@ function redraw() {
             if (!jsondata.directed && dst_int_id != null) {
                 //undirected, also include data for other interface
                 retval.push( { 'node': dst_node, 'interface':  dst_int_id, 'target': src_node, 'link': link });
-                }
+            }
 
             return retval;
 
@@ -1126,7 +1139,9 @@ function redraw() {
 
     var interface_area = function(d) {
         interface_id = d.interface;
-        return d.node._interfaces[interface_id]['area'];
+        asn = d.node['asn'];
+        area = d.node._interfaces[interface_id]['area'];
+        return asn + "," + area;
     }
 
     interface_attr_groups = d3.nest().key( interface_area ).entries(interface_data);
@@ -1138,7 +1153,7 @@ function redraw() {
     var hull_stroke_width = function() {
 
         if (display_interfaces && overlay_id == "ospf") {
-            return 40;
+            return 25;
         }
         return 80;
 
@@ -1167,6 +1182,7 @@ function redraw() {
     groupings.transition()
         .duration(500)
         .attr("d", groupPath)
+        .style("stroke-width", hull_stroke_width)
 
         groupings.exit().transition()
         .duration(1000)
@@ -1185,9 +1201,7 @@ function redraw() {
 
     //TODO: filter the json data x and y ranges: store in nodes, and use this for the image plotting
 
-
-    
-        node_highlight = chart.selectAll(".node_highlight")
+    node_highlight = chart.selectAll(".node_highlight")
         .data(highlight_nodes, function(d) { return d.id;})
 
         node_highlight.enter().append("svg:rect")
@@ -1272,7 +1286,7 @@ function redraw() {
     });
 
 
-        var highlight_line = chart.selectAll(".highlight_line")
+    var highlight_line = chart.selectAll(".highlight_line")
         .data(highlight_edges)
 
         //line.enter().append("line")
@@ -1295,7 +1309,7 @@ function redraw() {
         //.style("fill", "rgb(113,119,254)")
         .style("fill", "none")
 
-    highlight_line.transition()
+        highlight_line.transition()
         .duration(500)
         .attr("d", graph_edge)
         .style("opacity", line_opacity)
@@ -1305,12 +1319,6 @@ function redraw() {
         .style("opacity",0)
         .remove();
 
-
-
-
-    //TODO: handle removing of interfaces
-
-    //TODO: handling if no interface id specified
     starting_circles = chart.selectAll(".starting_circle")
         .data(starting_hosts, function(d) { return d.id;})
 
@@ -1321,81 +1329,79 @@ function redraw() {
         .attr("cx", function(d) { return d.x + x_offset + icon_width/2 ; })
         .attr("cy", function(d) { return d.y + y_offset + icon_height/2; })
 
-
-    starting_circles.transition()
+        starting_circles.transition()
         .attr("r", 60)
         .style("opacity",0)
         .duration(4000);
 
-
-    //console.log(interface_data);
-
     interface_icons = chart.selectAll(".interface_icon")
         //.data(interface_data) //TODO: check if need to provide an index
         //TODO: check if should return tuple of interface, node for uniqueness (esp for switching overlays)
-        .data(interface_data, function(d) { return d.interface;})
-        
+        .data(interface_data, function(d) { return (d.node, d.interface);})
+
         var highlight_interfaces = function(d) {
             interfaces = d3.selectAll(".interface_icon");
-            //interfaces.filter(
-
         }
 
-        interface_icons.enter().append("svg:rect")
-            .attr("class", "interface_icon")
-            .attr("width", interface_width)
-            .attr("height", interface_height)
-            .attr("x", interface_x)
-            .attr("y", interface_y)
+    interface_icons.enter().append("svg:rect")
+        .attr("class", "interface_icon")
+        .attr("width", interface_width)
+        .attr("height", interface_height)
+        .attr("x", interface_x)
+        .attr("y", interface_y)
+        .style("opacity", 0)
 
-            interface_icons
-            //TODO: look if can return multiple attributes, ie x and y, from the same function, ie calculation
-            .attr("fill", "rgb(6,120,155)")
+        interface_icons
+        //TODO: look if can return multiple attributes, ie x and y, from the same function, ie calculation
+        .attr("fill", "rgb(6,120,155)")
+        .style("opacity", 0)
 
-            .on("mouseover", function(d){
-                highlight_interfaces(d);
-                d3.select(this).style("stroke", "orange");
-                d3.select(this).style("fill", "yellow");
-                d3.select(this).style("stroke-width", "2");
-                d3.select(this).attr("marker-end", "");
-            })
-        .on("mouseout", function(){
+        .on("mouseover", function(d){
+            highlight_interfaces(d);
+            d3.select(this).style("stroke", "orange");
+            d3.select(this).style("fill", "yellow");
             d3.select(this).style("stroke-width", "2");
-            d3.select(this).style("stroke", "none");
-            d3.select(this).style("fill", "rgb(6,120,155)");
-            //d3.select(this).attr("marker-end", marker_end);
+            d3.select(this).attr("marker-end", "");
         })
+    .on("mouseout", function(){
+        d3.select(this).style("stroke-width", "2");
+        d3.select(this).style("stroke", "none");
+        d3.select(this).style("fill", "rgb(6,120,155)");
+        //d3.select(this).attr("marker-end", marker_end);
+    })
 
-        $('.interface_icon').tipsy({ 
-            //based on http://bl.ocks.org/1373263
-            gravity: 'w', 
-        html: true, 
-        title: function() {
-            var d = this.__data__
-            return interface_info(d); 
-        }
-        });
+    $('.interface_icon').tipsy({ 
+        //based on http://bl.ocks.org/1373263
+        gravity: 'w', 
+    html: true, 
+    title: function() {
+        var d = this.__data__
+        return interface_info(d); 
+    }
+    });
 
-        interface_icons.transition()
-            .attr("x", interface_x)
-            .attr("y", interface_y)
-            .duration(500);
+    interface_icons.transition()
+        .attr("x", interface_x)
+        .attr("y", interface_y)
+        .style("opacity", interface_opacity)
+        .duration(500);
 
-        interface_icons.exit().transition()
-            .duration(500)
-            .style("opacity",0)
-            .remove();
+    interface_icons.exit().transition()
+        .duration(500)
+        .style("opacity",0)
+        .remove();
 
-        interface_labels = chart.selectAll(".interface_label")
+    interface_labels = chart.selectAll(".interface_label")
         .data(interface_data, function(d) { return d.interface;})
-        
+
         interface_labels.enter().append("text")
         .attr("x", interface_x)
         .attr("y", interface_y)
         .attr("class", "interface_label")
         .attr("text-anchor", "middle") 
         .attr("font-family", "helvetica") 
-        .attr("font-size", "small") 
+        //.attr("font-size", "small") 
+        .attr("font-size", 8) 
 
         //TODO: use a general accessor for x/y of nodes
         interface_labels 
@@ -1403,7 +1409,7 @@ function redraw() {
         .attr("dy", -interface_height + 3) // vertical-align: middle
         .text(interface_label);
 
-        interface_labels.transition()
+    interface_labels.transition()
         .attr("x", interface_x)
         .attr("y", interface_y)
         .duration(500)
@@ -1413,45 +1419,44 @@ function redraw() {
         .style("opacity",0)
         .remove();
 
-        //Link labels
+    //Link labels
+    link_labels = chart.selectAll(".link_label")
+        .data(jsondata.links, edge_id)
 
-        link_labels = chart.selectAll(".link_label")
-            .data(jsondata.links, edge_id)
+        link_labels.enter().append("text")
+        .attr("x",link_label_x)
+        .attr("y", link_label_y )
+        .attr("class", "link_label")
+        .attr("text-anchor", "middle") 
+        .attr("font-family", "helvetica") 
+        .attr("font-size", "small") 
 
-            link_labels.enter().append("text")
-            .attr("x",link_label_x)
-            .attr("y", link_label_y )
-            .attr("class", "link_label")
-            .attr("text-anchor", "middle") 
-            .attr("font-family", "helvetica") 
-            .attr("font-size", "small") 
-
-            //TODO: use a general accessor for x/y of nodes
-            link_labels 
-            .attr("dx", 0) // padding-right
-            .attr("dy", 0) // vertical-align: middle
-            .text(function (d) {
-                if (edge_group_id == "_interfaces") {
-                    retval = "";
-                    for (attr in d[edge_group_id]) {
-                        retval += "(" + attr + ", " + d[edge_group_id][attr] + ")";
-                    }
-                    return retval;
+        //TODO: use a general accessor for x/y of nodes
+        link_labels 
+        .attr("dx", 0) // padding-right
+        .attr("dy", 0) // vertical-align: middle
+        .text(function (d) {
+            if (edge_group_id == "_interfaces") {
+                retval = "";
+                for (attr in d[edge_group_id]) {
+                    retval += "(" + attr + ", " + d[edge_group_id][attr] + ")";
                 }
-                return d[edge_group_id];
-            });
+                return retval;
+            }
+            return d[edge_group_id];
+        });
 
-        link_labels.transition()
-            .attr("x",link_label_x)
-            .attr("y", link_label_y )
-            .duration(500)
+    link_labels.transition()
+        .attr("x",link_label_x)
+        .attr("y", link_label_y )
+        .duration(500)
 
-            link_labels.exit().transition()
-            .duration(1000)
-            .style("opacity",0)
-            .remove();
+        link_labels.exit().transition()
+        .duration(1000)
+        .style("opacity",0)
+        .remove();
 
-        var node_id = function(d) {
+    var node_id = function(d) {
         return d.label + d.network;
     }
 
@@ -1501,10 +1506,6 @@ function redraw() {
         }
     });
 
-
-
-
-
     device_labels = chart.selectAll(".device_label")
         .data(nodes, node_id)
 
@@ -1534,7 +1535,6 @@ function redraw() {
         .style("opacity",0)
         .remove();
     //});
-
         }
 
 function redraw_paths() {
@@ -1579,7 +1579,7 @@ function redraw_paths() {
         .style("opacity", 50)
         ;
 
-        path2.exit().transition()
+    path2.exit().transition()
         .duration(2000)
         .style("opacity",0)
         .remove();
